{-------------------------------------------------------------------------------
  generic.Calc
 -------------------------------------------------------------------------------
  Creazione    : 17.01.1999
  Modifica     : 20.01.1999
  Versione     : 1.0.2
  Autore       : Riccardo "Rico" Pareschi (on a D.G. Barabash idea)
 -------------------------------------------------------------------------------
 20.01.1999 - first release

 ------------------------------------------------------------------------------}
unit Calc;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
     Buttons, ExtCtrls;


const
  DisplayDigits = 15;   // Number of digits in calculator display
  ZeroStr : string = '0';

type
  TCalcState = (csFirst, csValid, csError);

  TCalculator = class(TForm)
    GroupBox1: TGroupBox;
    Button0: TSpeedButton;
    Button1: TSpeedButton;
    Button2: TSpeedButton;
    Button3: TSpeedButton;
    Button4: TSpeedButton;
    Button5: TSpeedButton;
    Button6: TSpeedButton;
    Button7: TSpeedButton;
    Button8: TSpeedButton;
    Button9: TSpeedButton;
    PointButton: TSpeedButton;
    PercentButton: TSpeedButton;
    EqualButton: TSpeedButton;
    PlusButton: TSpeedButton;
    MinusButton: TSpeedButton;
    MultiplyButton: TSpeedButton;
    DivideButton: TSpeedButton;
    ClearButton: TSpeedButton;
    BackButton: TSpeedButton;
    NegativeButton: TSpeedButton;
    DisplayPanel: TPanel;
    SpeedButton1: TSpeedButton;
    CopiaSB: TSpeedButton;
    ExitSB: TSpeedButton;
    EuroSB: TSpeedButton;
    EuroLbl: TLabel;
    {- Hand crafted functions }
    procedure SetNumber(Value : string);
    procedure DoCalc(Sender: TObject);
    procedure NumberButtonClick(Sender: TObject);

    {- Rad functions }
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure NegativeButtonClick(Sender: TObject);
    procedure BackButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure PointButtonClick(Sender: TObject);
    procedure CopiaSBClick(Sender: TObject);
    procedure ExitSBClick(Sender: TObject);
    procedure EuroSBClick(Sender: TObject);
  private
    { Private declarations }
    Calcresult,
    CalcEuroTo: double;
    IsEuro: boolean;
  public
    { Public declarations }
    CalcState: TCalcState;
    Operator: Char;
    Operand: Extended;
    Negative: Boolean;
    Number: string;
  end;



var
  Calculator: TCalculator;

  function CalcExec(Euro: Double): Double;

implementation
{$R *.DFM}

{------------------------------------------------------------------------------}
{ HAND CRAFTED                                                                 }
{------------------------------------------------------------------------------}


{-------------------------------------------------------------------------------
}
procedure TCalculator.SetNumber(Value : string);
begin
  Number := Value;
  if Negative then Value := '-' + Value;
  DisplayPanel.Caption := Value;
end; { SetNumber }

{-------------------------------------------------------------------------------
  trap all digits as single event handler
}
procedure TCalculator.NumberButtonClick(Sender: TObject);
var
  C : Char;
begin
  if CalcState = csError then Exit;

  C := (Sender as TSpeedButton).Name[Length((Sender as TSpeedButton).Name)];

  if CalcState = csFirst then
     begin
       CalcState := csValid;
       Negative := False;
       Number := C;
     end
  else if Length(Number) < DisplayDigits then
     begin
       if Number = ZeroStr then
         Number := C
       else
         Number := Number + C;
     end;
  DisplayPanel.Caption := Number;
end; { NumberButtonClick }

{-------------------------------------------------------------------------------
  calculation method
}
procedure TCalculator.DoCalc(Sender: TObject);
var
  Value : Extended;
  Key : Char;

  procedure Error; // Sets error state
  begin
    CalcState := csError;
    DisplayPanel.Caption := 'Errore';
  end; { Error }

begin
  if CalcState = csError then Exit;

  // First char of button caption
  Key := (Sender as TSpeedButton).Caption[1];

  Value := StrToFloat(Number);
  if Negative then Value := -Value;

  if CalcState = csValid then
  begin
    CalcState := csFirst;

    if Key = '%' then
      case Operator of
        '+' : Value := Operand * Value / 100;
        '-' : Value := Operand * Value / 100;
        '*' : Value := Value / 100;
        '/' : Value := Value / 100;
      end;

    case Operator of
      '+' : Value := Operand + Value;
      '-' : Value := Operand - Value;
      '*' : Value := Operand * Value;
      '/' : if Value = 0 then begin
               Error;
               Exit;
            end else
               Value := Operand / Value;
    end;
  end; // CalcState

  Operand := Value;
  Operator := Key;

  Negative := Value < 0;
  if Negative then Value := -Value;
  Number := LowerCase(FloatToStr(Value));

  if Length(Number) > DisplayDigits then
     Error
  else SetNumber(Format('%f',[Value]));   
end; { DoCalc }


{------------------------------------------------------------------------------}
{ RAD FUNCTIONS                                                                }
{------------------------------------------------------------------------------}


{-------------------------------------------------------------------------------
  trap user pressed keys symbols
}
procedure TCalculator.FormKeyPress(Sender: TObject; var Key: Char);
begin
  case UpCase(Key) of
    '+' : DoCalc(PlusButton);
    '-' : DoCalc(MinusButton);
    '*' : DoCalc(MultiplyButton);
    '/' : DoCalc(DivideButton);
    '%' : DoCalc(PercentButton);
    '=' : DoCalc(EqualButton);
    '0'..'9' : NumberButtonClick(FindComponent('Button' + Key));
    '_' : NegativeButtonClick(Sender);
    #8  : BackButtonClick(Sender);
    'C' : ClearButtonClick(Sender);
  else
    if Key = DecimalSeparator then PointButton.Click;
  end;
end; { FormKeyPress }

{-------------------------------------------------------------------------------
  trap Return key as EqualButton
}
procedure TCalculator.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
  begin
    Key := 0;
    DoCalc(EqualButton);
  end;
end; { FormKeyDown }

{-------------------------------------------------------------------------------
}
procedure TCalculator.NegativeButtonClick(Sender: TObject);
begin
  if CalcState = csError then Exit;
  Negative := not Negative;
  Number := Number;
  DisplayPanel.Caption := Number;
end; { NegativeButtonClick }

{-------------------------------------------------------------------------------
}
procedure TCalculator.BackButtonClick(Sender: TObject);
begin
  if CalcState = csError then Exit;
  if CalcState = csValid then
  begin
    Number := Copy(Number, 1, Length(Number) - 1);
    if Number = '' then Number := ZeroStr;
  end;
  DisplayPanel.Caption := Number;
end; { BackButtonClick }

{-------------------------------------------------------------------------------
}
procedure TCalculator.ClearButtonClick(Sender: TObject);
begin
  CalcState := csFirst;
  Operand := 0;
  Operator := '=';
  Negative := False;
  Number := ZeroStr;
  DisplayPanel.Caption := Number;
end; { ClearButtonClick }

{-------------------------------------------------------------------------------
}
procedure TCalculator.PointButtonClick(Sender: TObject);
begin
  if CalcState = csError then Exit;
  if CalcState = csFirst then
  begin
    CalcState := csValid;
    Negative := False;
    Number := ZeroStr;
  end;
  if (Pos(DecimalSeparator, Number) = 0) and (Length(Number) < DisplayDigits) then
    Number := Number + DecimalSeparator;
  DisplayPanel.Caption := Number;
end; { PointButtonClick }

{-------------------------------------------------------------------------------
}
procedure TCalculator.CopiaSBClick(Sender: TObject);
begin
  CalcResult := StrToFloat(DisplayPanel.Caption);
  ModalResult := mrOk;
end; { CopiaSBClick }

{-------------------------------------------------------------------------------
}
procedure TCalculator.ExitSBClick(Sender: TObject);
begin
  CalcResult := 0.0;
  ModalResult := mrCancel;
end; { ExitSBClick }

{-------------------------------------------------------------------------------
}
procedure TCalculator.EuroSBClick(Sender: TObject);
var ValTmp: real;
begin
  ValTmp := StrToFloat(DisplayPanel.Caption);
  if IsEuro then
     DisplayPanel.Caption := Format('%f',[ValTmp*CalcEuroTo])
  else DisplayPanel.Caption := Format('%f',[ValTmp/CalcEuroTo]);
  IsEuro := NOT IsEuro;
  EuroLbl.Visible := IsEuro;
end; { EuroSBClick }

{------------------------------------------------------------------------------}
{ EXEC DIALOG FUNCTION                                                         }
{------------------------------------------------------------------------------}

{-------------------------------------------------------------------------------
 to call a Dll dialog is a good solution to incapsulate all the life of the
 form so we can return a value as function result.
}
function CalcExec(Euro: Double): Double;
begin
  result := 0.0;
  // create the dialog

  Calculator := TCalculator.Create(Application);
  try
    with Calculator do begin
      //  init the data
      IsEuro := False;
      EuroLbl.Visible := IsEuro;
      CalcEuroTo := Euro;
      // show the dialog
      if ShowModal = mrOk then begin
         result := CalcResult;
      end;
    end; // with
  finally
    Calculator.Free;  // free the memory
  end;
end; { CalcExec }



end.
