unit MainDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls, Buttons, ComCtrls, ClipBrd;

type
  TCalcState = (csFirst, csValid, csError);

  TEuroCalcForm = class(TForm)
    GroupBox1: TGroupBox;
    Button0: TSpeedButton;
    Button1: TSpeedButton;
    Button2: TSpeedButton;
    Button3: TSpeedButton;
    Button4: TSpeedButton;
    Button5: TSpeedButton;
    Button6: TSpeedButton;
    Button7: TSpeedButton;
    Button8: TSpeedButton;
    Button9: TSpeedButton;
    PointButton: TSpeedButton;
    PercentButton: TSpeedButton;
    EqualButton: TSpeedButton;
    PlusButton: TSpeedButton;
    MinusButton: TSpeedButton;
    MultiplyButton: TSpeedButton;
    DivideButton: TSpeedButton;
    ClearButton: TSpeedButton;
    BackButton: TSpeedButton;
    NegativeButton: TSpeedButton;
    CopiaSB: TSpeedButton;
    ExitSB: TSpeedButton;
    EuroSB: TSpeedButton;
    EuroLbl: TLabel;
    DisplayPanel: TPanel;
    SpeedButton1: TSpeedButton;
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    {- Hand crafted functions }
    procedure SetNumber(Value : string);
    procedure DoCalc(Sender: TObject);
    procedure NumberButtonClick(Sender: TObject);

    {- Rad functions }
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure NegativeButtonClick(Sender: TObject);
    procedure BackButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure PointButtonClick(Sender: TObject);
    procedure CopiaSBClick(Sender: TObject);
    procedure ExitSBClick(Sender: TObject);
    procedure EuroSBClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Private declarations }
    CalcEuroTo: double;
    IsEuro: boolean;
    IsInfo: boolean;
  public
    { Public declarations }
    CalcState: TCalcState;
    Operator: Char;
    Operand: Extended;
    Negative: Boolean;
    Number: string;
  end;

var
  EuroCalcForm: TEuroCalcForm;

implementation

uses Calc;

{$R *.DFM}

{------------------------------------------------------------------------------}
{ HAND CRAFTED                                                                 }
{------------------------------------------------------------------------------}

//  clipboard

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.SetNumber(Value : string);
begin
  Number := Value;
  if Negative then Value := '-' + Value;
  DisplayPanel.Caption := Value;
end; { SetNumber }

{-------------------------------------------------------------------------------
  trap all digits as single event handler
}
procedure TEuroCalcForm.NumberButtonClick(Sender: TObject);
var
  C : Char;
begin
  if CalcState = csError then Exit;

  C := (Sender as TSpeedButton).Name[Length((Sender as TSpeedButton).Name)];

  if CalcState = csFirst then
     begin
       CalcState := csValid;
       Negative := False;
       Number := C;
     end
  else if Length(Number) < DisplayDigits then
     begin
       if Number = ZeroStr then
         Number := C
       else
         Number := Number + C;
     end;
  DisplayPanel.Caption := Number;
end; { NumberButtonClick }

{-------------------------------------------------------------------------------
  calculation method
}
procedure TEuroCalcForm.DoCalc(Sender: TObject);
var
  Value : Extended;
  Key : Char;

  procedure Error; // Sets error state
  begin
    CalcState := csError;
    DisplayPanel.Caption := 'Errore';
  end; { Error }

begin
  if CalcState = csError then Exit;

  // First char of button caption
  Key := (Sender as TSpeedButton).Caption[1];

  Value := StrToFloat(Number);
  if Negative then Value := -Value;

  if CalcState = csValid then
  begin
    CalcState := csFirst;

    if Key = '%' then
      case Operator of
        '+' : Value := Operand * Value / 100;
        '-' : Value := Operand * Value / 100;
        '*' : Value := Value / 100;
        '/' : Value := Value / 100;
      end;

    case Operator of
      '+' : Value := Operand + Value;
      '-' : Value := Operand - Value;
      '*' : Value := Operand * Value;
      '/' : if Value = 0 then begin
               Error;
               Exit;
            end else
               Value := Operand / Value;
    end;
  end; // CalcState

  Operand := Value;
  Operator := Key;

  Negative := Value < 0;
  if Negative then Value := -Value;
  Number := LowerCase(FloatToStr(Value));

  if Length(Number) > DisplayDigits then
     Error
  else SetNumber(Format('%f',[Value]));
end; { DoCalc }


{------------------------------------------------------------------------------}
{ RAD FUNCTIONS                                                                }
{------------------------------------------------------------------------------}


{-------------------------------------------------------------------------------
  trap user pressed keys symbols
}
procedure TEuroCalcForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  case UpCase(Key) of
    '+' : DoCalc(PlusButton);
    '-' : DoCalc(MinusButton);
    '*' : DoCalc(MultiplyButton);
    '/' : DoCalc(DivideButton);
    '%' : DoCalc(PercentButton);
    '=' : DoCalc(EqualButton);
    '0'..'9' : NumberButtonClick(FindComponent('Button' + Key));
    '_' : NegativeButtonClick(Sender);
    #8  : BackButtonClick(Sender);
    'C' : ClearButtonClick(Sender);
  else
    if Key = DecimalSeparator then PointButton.Click;
  end;
end; { FormKeyPress }

{-------------------------------------------------------------------------------
  trap Return key as EqualButton
}
procedure TEuroCalcForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
  begin
    Key := 0;
    DoCalc(EqualButton);
  end;
end; { FormKeyDown }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.NegativeButtonClick(Sender: TObject);
begin
  if CalcState = csError then Exit;
  Negative := not Negative;
  Number := Number;
  DisplayPanel.Caption := Number;
end; { NegativeButtonClick }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.BackButtonClick(Sender: TObject);
begin
  if CalcState = csError then Exit;
  if CalcState = csValid then
  begin
    Number := Copy(Number, 1, Length(Number) - 1);
    if Number = '' then Number := ZeroStr;
  end;
  DisplayPanel.Caption := Number;
end; { BackButtonClick }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.ClearButtonClick(Sender: TObject);
begin
  CalcState := csFirst;
  Operand := 0;
  Operator := '=';
  Negative := False;
  Number := ZeroStr;
  DisplayPanel.Caption := Number;
end; { ClearButtonClick }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.PointButtonClick(Sender: TObject);
begin
  if CalcState = csError then Exit;
  if CalcState = csFirst then
  begin
    CalcState := csValid;
    Negative := False;
    Number := ZeroStr;
  end;
  if (Pos(DecimalSeparator, Number) = 0) and (Length(Number) < DisplayDigits) then
    Number := Number + DecimalSeparator;
  DisplayPanel.Caption := Number;
end; { PointButtonClick }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.CopiaSBClick(Sender: TObject);
begin
  Clipboard.Clear;
  Clipboard.SetTextBuf(PChar(DisplayPanel.Caption));
end; { CopiaSBClick }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.ExitSBClick(Sender: TObject);
begin
  Close;
end; { ExitSBClick }

{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.EuroSBClick(Sender: TObject);
var ValTmp: real;
begin
  if (DisplayPanel.Caption <> '') then begin
     ValTmp := StrToFloat(DisplayPanel.Caption);
     if IsEuro then
        DisplayPanel.Caption := Format('%f',[ValTmp*CalcEuroTo])
     else DisplayPanel.Caption := Format('%f',[ValTmp/CalcEuroTo]);
     IsEuro := NOT IsEuro;
     EuroLbl.Visible := IsEuro;
  end;
end; { EuroSBClick }


{-------------------------------------------------------------------------------
}
procedure TEuroCalcForm.FormCreate(Sender: TObject);
begin
  // Euro settings
  CalcEuroTo := 1936.27;
  IsEuro := False;
  EuroLbl.Visible := IsEuro;

  // Info settings
  IsInfo := False;
  ClientWidth := 152;

  // Generic settings
  DisplayPanel.caption := '';
end; { FormCreate }


procedure TEuroCalcForm.SpeedButton1Click(Sender: TObject);
begin
  if NOT IsInfo then begin
     ClientWidth := 260;
  end else begin
     ClientWidth := 152;
  end;
  IsInfo := NOT IsInfo;
end;

end.
